#!/usr/bin/perl
#  uncleared-txn.pl - Prints a report of the uncleared transactions sorted 
#                     by date
#
#  Written by Kevin L. McWhirter (klmcw@crl.com).  Started May 18, 1997
#
#  Copyright (C) 1994 - 1997  Curtis L. Olson  - curt@sledge.mn.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id: uncleared-txn.pl,v 2.4 1999/03/15 17:20:02 curt Exp $
# (Log is kept at end of this file)


package CBB;

use strict;    # don't take no guff


my($cbb_incl_dir, $temp);
my($account, @account_list, $name, $key, $trans, %ALLTRANS);
my($firstdate, $lastdate);
my($firstday, $firstmon, $firstyear, $lastday, $lastmon, $lastyear);
my($totalmonths);
my($junk, $result);
my($date_fmt, $date, $todate, $fromdate);

# specify the installed location of the necessary pieces.
BEGIN{
$CBB::cbb_incl_dir = "/usr/lib/cbb";
unshift(@INC, $CBB::cbb_incl_dir);
}

require "common.pl";
require "reports.pl";
require "engine.pl";
require "memorized.pl";


($#ARGV >= 0) || 
    die "Usage: report [ -from mm/dd/[yy]yy ] [ -to mm/dd/[yy]yy ] accounts";


# process arguments

($date_fmt, $fromdate, $todate, @account_list) = &process_rep_args();

if ( $fromdate eq "all" ) {
    $fromdate = "";
}

if ( $todate eq "all" ) {
    $todate = "";
}

# print "'$fromdate' '$todate' '@account_list'\n";

%ALLTRANS = ();

# load all matching transactions from all specified accounts (ignoring
# those that are outside the specified date range)

foreach $account ( @account_list ) {
    $name = &file_basename($account);

    # open the account
    (&load_trans($account) eq "ok") || die "Cannot open account:  $account";

    $result = &first_trans();
    while ( $result ne "none" ) {
        ($key, $trans) = split(/\t/, $result, 2);
        my($date, $check, $desc, $debit, $credit, $cat, $com, $cleared,
	   $junk) = split(/\t/, $trans, 9);

	if ( ($firstdate eq "") || ($date < $firstdate) ) {
	    $firstdate = $date;
	}

	if ( ($lastdate eq "") || ($date > $lastdate) ) {
	    $lastdate = $date;
	}

        if ( (($fromdate == 0) || ($fromdate <= $date)) && 
            (($todate == 0) || ($todate >= $date)) &&
	    ($cleared ne "x") ) {
	    $ALLTRANS{"$key$name"} = $trans;
        }

        $result = &next_trans();
    }
}

# determine number of months in range
if (($fromdate != 0) && ($fromdate > $firstdate)) {
    $firstdate = $fromdate;
}
if (($todate != 0) && ($todate < $lastdate)) {
    $lastdate = $todate;
}
($firstyear,$firstmon,$firstday) = $firstdate =~ /(\d\d\d\d)(\d\d)(\d\d)/;
($lastyear,$lastmon,$lastday) = $lastdate =~ /(\d\d\d\d)(\d\d)(\d\d)/;

$totalmonths = ($lastyear - $firstyear) * 12;
$totalmonths += ($lastmon - $firstmon);
$totalmonths++;

if ( $date_fmt == 1 ) {
    print "Uncleared Transactions for $firstmon/$firstday/$firstyear".
	" to $lastmon/$lastday/$lastyear.\n";
} else {
    print "Uncleared Transactions for $firstday.$firstmon.$firstyear".
	" to $lastday.$lastmon.$lastyear.\n";
}
print "    (Range is $totalmonths months.)\n";
print "\n";

# sort and print
foreach $key (sort (keys %ALLTRANS) ) {
    # print $ALLTRANS{$key} . "\n";
    &format_line("$ALLTRANS{$key}");
}


sub format_line {
    my($date, $check, $desc, $debit, $credit, $cat, $com, $cleared,
          $total) = split(/\t/, $_[0]);
    my($year,$mon,$day) = $date =~ /(\d\d\d\d)(\d\d)(\d\d)/;
    my($nicedate, $cutdesc, $cutcom, $nicecat);

    $year = substr($year, 2, 4);

    if ( $date_fmt == 1 ) {
	$nicedate = "$mon/$day/$year";
    } else {
	$nicedate = "$day.$mon.$year";
    }
    $cutdesc = substr($desc, 0, 15);
    $cutcom = substr($com, 0, 15);
    $nicecat = substr($cat, 0, 9);
    if ( substr($cat, 0, 1) eq "|" ) {
        $nicecat = "-Splits-";
    }

    printf("%5s  %-8s  %-15s  %9.2f  %9.2f  %-1s  %9.2f\n",
          $check, $nicedate, $cutdesc, $debit, $credit, $cleared, $total);
    printf("%5s  %-8s  %-15s  %-9s\n\n", "", "", $cutcom, $nicecat);
}

# ----------------------------------------------------------------------------
# $Log: uncleared-txn.pl,v $
# Revision 2.4  1999/03/15 17:20:02  curt
# @INC() patches contributed by Guy Geens <ggeens@iname.com>
#
# Revision 2.3  1999/01/18 13:51:06  curt
# Check for the existance of ~/.cbbmath before trying to load it.
#
# Fixed a small bug in import_qif() relating to the logic of properly
# reconstructing splits.
#
# Added a script to convert MYM files to CBB.  Contributed by Brian
# <briank@hex.net>
#
# Added an Emacs "edb" interface to the contrib section.  Provided by
# Bob Newell <bnewell@alum.mit.edu>
#
# Preserve owner, group, and permissions of CBB files
#
# Added a loan_recur.pl script from Michel Verdier which does the same
# thing as loan.pl, but also generates recuring transactions for the
# payments.
#
# Patch from Michel Verdier to help recur.pl better handle recurring
# transfer transactions.
#
# Added initial support for binding an arbitrary math function to the
# debit and credit fields in the main transaction editor section.  This
# is useful for doing things like currency conversions.  You can enter
# in one currency and then use a hot key to convert to your "canonical"
# currency.
#
# Revision 2.2  1998/08/14 14:28:52  curt
# Added desc-pie graph.
# Added option to eliminate splash screen.
# Other misc. tweaks and bug fixes.
#
# Revision 2.1  1997/07/02 18:31:56  curt
# Add an "Uncleared transaction" report.
#
